#include <qpid/client/Dispatcher.h>
#include <qpid/client/Connection.h>
#include <qpid/client/Session.h>
#include <qpid/client/Message.h>
#include <qpid/client/SubscriptionManager.h>

#include <unistd.h>
#include <cstdlib>
#include <iostream>

using namespace qpid::client;
using namespace qpid::framing;

using namespace std;



class Listener 
  : public MessageListener
{
  private:
    SubscriptionManager& subscriptions;
    int count;
    string q_name;

    char timestamp[1000];

    void update_timestamp();

  public:
    Listener ( SubscriptionManager& subscriptions, char const * queue_name );
    void received(Message& message);
};



void
Listener::update_timestamp ( )
{
  time_t rawtime;
  time ( & rawtime );
  struct tm * timeinfo = localtime ( &rawtime );
  int seconds;

  // Prints the "second number" for the current momment.
  seconds = timeinfo->tm_hour * 3600 +
            timeinfo->tm_min  * 60   +
            timeinfo->tm_sec;

  sprintf ( timestamp, "%d", seconds );
}




Listener::Listener ( SubscriptionManager& subs, char const * queue_name ) 
  : subscriptions(subs),
    count(0),
    q_name(queue_name)
{
}





void 
Listener::received ( Message& message ) 
{
  //cout << "Listener received: " << message.getData() << endl;
  if (message.getData() == "quit") 
  {
    update_timestamp ( );

    char file_name[1000];
    sprintf ( file_name, "./listeners_complete/%s", q_name.c_str() );
    FILE * fp = fopen ( file_name, "w" );
    fprintf ( fp, "%s\n", timestamp );
    fclose ( fp );

    subscriptions.cancel(message.getDestination());
  }
  else
  {
    ++ count;
  }
}





int 
main ( int argc, char** argv ) 
{
  const char* host   =      argv[1];
  int port           = atoi(argv[2]);
  char const* q_name =      argv[3];

  Connection connection;
  Message msg;

  connection.open(host, port);
  Session session =  connection.newSession();

  SubscriptionManager subscriptions(session);

  // Create a listener and subscribe it to the queue named "message_queue"
  Listener listener(subscriptions, q_name);

  subscriptions.subscribe(listener, q_name );

  cerr << "Listener subscribed to " << q_name << "\n";

  // Deliver messages until the subscription is cancelled
  // by Listener::received()
  subscriptions.run();

  connection.close();

  system ( "date" );
  return 0;
}





